// Net.java
// J2SE 1.5:
//   Use generics for Vector
//   Use for-each

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class Net extends JPanel
{
  private GFrame f = new GFrame();
  private Vector<Item> items = new Vector<Item>();
  private int iHot = -1; // Index of hot item (-1: no hot item)

  public Net()
  {
    f.setResizable(false);
    f.showComponent(this);
    f.getOffG2D().setColor(Color.black);
    f.getOffG2D().drawString(
      "Use right click to create new nodes", 10, 40);
    f.getOffG2D().drawString("Use left button to move nodes",
                             10, 60);
    repaint();

    addMouseListener(new MouseAdapter()
    {
      public void mousePressed(MouseEvent evt)
      {
        if (!SwingUtilities.isLeftMouseButton(evt))
          return;
        Node p = null;
        for (int i = 0; i < items.size() && iHot == -1; i++)
        {
          if (items.elementAt(i) instanceof Node)
          {
            p = (Node)items.elementAt(i);
            if (p.isHot(evt.getX(), evt.getY()))
            {
              setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
              p.setLocation(evt.getX(), evt.getY());
              iHot = i;
            }
          }
        }

        // Move hot item at start of vector to get it first
        if (iHot != -1)
        {
          items.remove(iHot);
          items.add(0, p);
          iHot = 0;
        }
        drawItems();
      }

      public void mouseReleased(MouseEvent evt)
      {
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        iHot = -1; // No hot item anymore
      }

      public void mouseClicked(MouseEvent evt)
      {
        if (!SwingUtilities.isRightMouseButton(evt))
          return;
        Node n = new Node(evt.getX(), evt.getY());
        items.add(n);

        for (int i = 0; i < items.size() - 1; i++)  // without last element!
        {
          if (items.elementAt(i) instanceof Node)
            items.add(new Edge((Node)items.elementAt(i), n));
        }
        drawItems();
      }
    });

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseDragged(MouseEvent evt)
      {
        if (!SwingUtilities.isLeftMouseButton(evt))
          return;
        if (iHot != -1) // Hot item found
        {
          Node p = (Node)items.elementAt(iHot);
          p.setLocation(evt.getX(), evt.getY());
          drawItems();
        }
      }

      public void mouseMoved(MouseEvent evt)
      {
        Node p = null;
        boolean onHot = false;
        for (Item it : items)
        {
          if (it instanceof Node)
          {
            p = (Node)it;
            if (p.isHot(evt.getX(), evt.getY()))
              onHot = true;
          }
        }

        if (onHot)
          setCursor(new Cursor(Cursor.HAND_CURSOR));
        else
          setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
      }
    });
  }

  private void drawItems()
  {
    f.clearBuf();
    Graphics2D g2D = f.getOffG2D();

    // Draw edges first, to put the nodes on top
    for (Item it : items)
      if (it instanceof Edge)
        it.draw(g2D);
    for (Item it : items)
      if (it instanceof Node)
        it.draw(g2D);
    repaint();
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    BufferedImage bi = f.getBufferedImage();
    Graphics2D g2D = (Graphics2D)g;
    g2D.drawImage(bi, 0, 0, this);
  }

  public static void main(String[] args)
  {
    new Net();
  }
}
