// GuiEx8.java
// If GPane's drawing methods are used in the constructor
// they must run in a separate thread because they execute
// only after the frame is visible (after the constructor
// terminates).

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import ch.aplu.util.*;

public class GuiEx8
{
  private JFrame frame = new JFrame("GUI with GPanel");
  private JPanel contentPane;
  private JPanel navigationPane = new JPanel();
  private JPanel graphPane = new JPanel();
  private GPane gPane = new GPane();
  private JButton drawBtn = new JButton("Draw");
  private JButton clearBtn = new JButton("Clear");

  public GuiEx8()
  {
    frame.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent evt)
      {
        gPane.dispose();
        frame.setVisible(false);
        frame.dispose();
        System.exit(0);
      }
    });
    init();
    frame.pack();
    frame.setVisible(true);

    new Thread()
    {
      public void run()
      {
        drawRectangle();
      }
    }.start();
  }

  private void init()
  {
    contentPane = (JPanel)frame.getContentPane();
    contentPane.add(navigationPane, BorderLayout.WEST);
    contentPane.add(graphPane, BorderLayout.CENTER);
    navigationPane.setBackground(Color.lightGray);
    navigationPane.add(drawBtn);
    navigationPane.add(clearBtn);

    graphPane.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Graph Pane"),
        BorderFactory.createEmptyBorder(5, 5, 5, 5)));
    graphPane.setPreferredSize(new Dimension(300, 200));
    graphPane.setLayout(new BorderLayout());
    graphPane.setBackground(Color.white);
    graphPane.add(gPane);

    gPane.bgColor(Color.cyan);
    gPane.window(0, 100, 0, 100);

    drawBtn.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
        draw();
      }
    });

    clearBtn.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
        gPane.clear();
        drawRectangle();
      }
    });
  }

  private void drawRectangle()
  {
    gPane.move(0, 0);
    gPane.draw(100, 0);
    gPane.draw(100, 100);
    gPane.draw(0, 100);
    gPane.draw(0, 0);
  }

  private void draw()
  {
    for (double x = 0; x < 100; x += 5)
      gPane.line(x, 0, 0, 100 - x);
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new GuiEx8();
      }
    });
  }
}
