// FourierEx2.java
// Compiles J2SE V5 up

import ch.aplu.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class FourierEx2 implements ActionListener
{
  enum State
  {
    IDLE, DRAWING, SELECTING, EXITING;
  }

  private State state = State.IDLE;
  private GPanel p;
  private JMenuItem drawItem, exitItem, selectItem;
  private SettingsDialog sDlg;
  private int funcNb = 0;

  public FourierEx2()
  {
    p = new GPanel("Fourier-Reihen", setMenu(),
                   -1, 4 * Math.PI + 1, -2, 2);
    sDlg = new SettingsDialog(p.getWindow());
    drawCoordinateSystem();

    new Thread()
    {
      public void run()
      {
        eventLoop();
      }
    }.start();
  }

  private void eventLoop()
  {
    // Event loop
    while (state != State.EXITING)
    {
      Thread.currentThread().yield(); //For better performance
      if (state == State.DRAWING)
      {
        p.clear();
        drawCoordinateSystem();
        FourierSeries fs = new FourierSeries(funcNb, 5);
        fs.drawElements(p, 0, 4 * Math.PI);
        fs.drawSum(p, 0, 4 * Math.PI);
        state = State.IDLE;
      }
      if (state == State.SELECTING)
      {
        boolean rc = sDlg.show(funcNb);
        if (rc)
          funcNb = sDlg.getChoice();
        state = State.IDLE;
      }
    }
    System.exit(0);
  }

  private JMenuBar setMenu()
  {
    JMenu fileMenu = new JMenu("File");
    drawItem = new JMenuItem("Draw");
    fileMenu.add(drawItem);
    drawItem.addActionListener(this);
    exitItem = new JMenuItem("Exit");
    fileMenu.add(exitItem);
    exitItem.addActionListener(this);

    JMenu settingMenu = new JMenu("Settings");
    selectItem = new JMenuItem("Select Function");
    settingMenu.add(selectItem);
    selectItem.addActionListener(this);

    JMenuBar menuBar = new JMenuBar();
    menuBar.add(fileMenu);
    menuBar.add(settingMenu);

    fileMenu.setMnemonic('F');
    settingMenu.setMnemonic('S');

    drawItem.setAccelerator(KeyStroke.getKeyStroke('D',
            InputEvent.ALT_MASK));
    exitItem.setAccelerator(KeyStroke.getKeyStroke('X',
            InputEvent.ALT_MASK));
    selectItem.setAccelerator(KeyStroke.getKeyStroke('O',
            InputEvent.ALT_MASK));

    return menuBar;
  }

  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() == drawItem)
      state = State.DRAWING;
    if (e.getSource() == exitItem)
      state = State.EXITING;
    if (e.getSource() == selectItem)
      state = State.SELECTING;
  }

  private void drawCoordinateSystem()
  {
    p.move(0, 0);
    p.draw(4 * Math.PI, 0);
    p.move(0, -2);
    p.draw(0, 2);
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new FourierEx2();
      }
    });
  }

}
