// PopSim.java
// Compiles J2SE V5 up

import java.awt.EventQueue;

public class PopSim
{
  protected static State state = State.RESET;
  private Population pop;
  private PopDialog dlg;
  private int nbPop = 40;
  private int nbSick = 2;
  private int genTime = 10;

  public PopSim()
  {
    dlg = new PopDialog();
    dlg.setTextField(0, "" + nbSick);
    dlg.setTextField(1, "" + genTime);
    new Thread()
    {
      public void run()
      {
        eventLoop();
      }
    }.start();
  }

  private void eventLoop()
  {
    while (state != State.EXIT)
    {
      switch (state)
      {
        case IDLE :
        case HALT :
          Monitor.putSleep();
          break;

        case INIT :
          dlg.enableInput(false);
          nbSick = dlg.getInputValue(0, 1, 40);
          genTime = dlg.getInputValue(1, 1, 100);
          if (nbSick == 0 || genTime == 0 || nbSick > nbPop)
          {
            dlg.showStatus("Ungltige Anfangswerte");
            dlg.enableInput(true);
            state = State.IDLE;
          }
          else
          {
            pop = new Population(nbPop, nbSick);
            dlg.showText(pop.getPopDescription());
            dlg.showStatus(pop.getPopStatus());
            if (nbSick < nbPop)
              state = State.GO;
            else
              state = State.HALT;
          }
          break;

        case GO :
          delay(10 * genTime);
          pop.nextGen();
          if (pop.getSick() == nbPop)
            state = State.HALT;
          dlg.showText(pop.getPopDescription());
          dlg.showStatus(pop.getPopStatus());
          break;

        case RESET :
          dlg.clear();
          dlg.enableInput(true);
          dlg.showStatus(
            "Anfangswerte eingeben (Anzahl Individuen: " +
            nbPop + ") ");
          state = State.IDLE;
          break;
      }
    }
    System.exit(0);
  }

  private void delay(int time)
  {
    try
    {
      Thread.currentThread().sleep(time);
    }
    catch (InterruptedException ex) {}
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new PopSim();
      }
    });
  }
}
