// Debug.java

import java.text.DateFormat;
import java.util.Date;
import java.io.*;

class Debug
{
  private static DateFormat df =
    DateFormat.getDateTimeInstance(DateFormat.SHORT,
                                   DateFormat.MEDIUM);

  public static final boolean traceLow =
    SharedConstants.debugLevel > SharedConstants.OFF;
  public static final boolean traceMedium =
    SharedConstants.debugLevel > SharedConstants.LOW;
  public static final boolean traceHigh =
    SharedConstants.debugLevel > SharedConstants.MEDIUM;

  public static void log(String msg, int debugLevel)
  {
    print(null, msg, false, debugLevel);
  }

  public static void log(String filename, String msg,
                         int debugLevel)
  {
    print(filename, msg, false, debugLevel);
  }

  public static void stamp(String msg, int debugLevel)
  {
    print(null, msg, true, debugLevel);
  }

  public static void stamp(String filename, String msg,
                           int debugLevel)
  {
    print(filename, msg, true, debugLevel);
  }

  // ------------- Non public method -------------
  private static void print(String filename, String msg,
                            boolean withDate, int debugLevel)
  {
    if (SharedConstants.debugLevel < debugLevel)
      return;
    PrintStream dest = null;
    boolean isFileOpen = false;
    try
    {
      if (filename == null || filename.equals(""))
        dest = System.out;
      else
      {
        dest = new PrintStream(
          new FileOutputStream(new File(filename), true)); // append
        isFileOpen = true;
      }
      if (withDate)
        msg = new String(df.format(new Date()) + ":  " + msg);
      dest.println(msg);
    }
    catch (IOException ex)
    {
      System.out.println(ex.getMessage());
    }
    if (isFileOpen)
      dest.close();
  }
}
