// EchoServer.java

import java.net.*;
import java.io.*;

public class EchoServer
{
  // --------------- Inner class SocketHandler -------------
  private class SocketHandler extends Thread
  {
    private Socket socket;

    public SocketHandler(Socket socket)
    {
      this.socket = socket;
      setDaemon(true);
      nbClients++;
      System.out.println("# of clients: " + nbClients);
    }

    public void run()
    {
      BufferedReader in = null;
      PrintWriter out = null;
      try
      {
        in = new BufferedReader(
          new InputStreamReader(socket.getInputStream()));
        out = new PrintWriter(socket.getOutputStream());
        String line;
        while ((line = in.readLine()) != null)
        {
          System.out.println("Received from " +
                             socket.getInetAddress() +
                             " : " + line);
          out.println(line);
          out.flush();
        }
      }
      catch (IOException ex) {}
      try
      {
        if (in != null)
          in.close();
        if (out != null)
          out.close();
        socket.close();
      }
      catch (IOException ex) {}
      System.out.println("Client socket closed");
      nbClients--;
      System.out.println("# of clients: " + nbClients);
    }
  }

  // --------------- Inner class ServerHandler -------------
  class ServerHandler extends Thread
  {
    private ServerSocket serverSocket;

    ServerHandler(ServerSocket serverSocket)
    {
      this.serverSocket = serverSocket;
      setDaemon(true);
    }

    public void run()
    {
      try
      {
        while (true)
        {
          Socket socket = serverSocket.accept();
          System.out.println("Connection to " +
                             socket.getInetAddress() +
                             " established");
          new SocketHandler(socket).start();
        }
      }
      catch (IOException ex)
      {
        isRunning = false;
      }
    }
  }

  // --------------- End of inner classes ------------------

  private static int port = 23; // Default telnet port
  private int maxNbClients = 3;
  private int nbClients = 0;
  private boolean isRunning = true;

  public EchoServer()
  {
    try
    {
      ServerSocket serverSocket = new ServerSocket(port);
      System.out.println("Server running on port " + port);
      new ServerHandler(serverSocket).start();
      while (isRunning && nbClients <= maxNbClients)
        Thread.currentThread().yield();
      serverSocket.close();
      System.out.println("Server socket closed");
    }
    catch (Exception ex)
    {
      System.out.println("Port already in use");
    }
    System.out.println("Host exiting...");
  }

  public static void main(String[] args)
  {
    if (args.length > 0)
      port = Integer.parseInt(args[0]);
    new EchoServer();
  }
}
