// LptPort.java

/*
  This software is part of the JEX (Java Exemplarisch) Utility Library.
  It is Open Source Free Software, so you may
    - run the code for any purpose
    - study how the code works and adapt it to your needs
    - integrate all or parts of the code in your own programs
    - redistribute copies of the code
    - improve the code and release your improvements to the public
  However the use of the code is entirely your responsibility.
 */


package ch.aplu.util;

public class LptPort
{
  /**
   * Open LPT port at specified base address, e.g. 0x378.
   * Return 0 if successful, -1 otherwise.
   */
  public native int open(int address);

  /**
   * Return true if port is open, false otherwise.
   */
  public native boolean isOpen();

  /**
   * Write lower byte of data into data port (at base address).
   * (Upper byte is ignored.)
   * Return -1 if port is not open, 0 otherwise
   */
  public native int writeData(int data);

  /**
   * Write lower byte of data into control port (at base address + 2).
   * (Upper byte is ignored.)
   * Return -1 if port is not open, 0 otherwise
   */
  public native int writeControl(int data);

  /**
   * Return content of data port (at base address) in lower byte.
   * (Upper byte is 0.)
   * Return -1 if port is not open.
   */
  public native int readData();

  /**
   * Return content of status port (at base address + 1) in lower byte.
   * (Upper byte is 0.)
   * Return -1 if port is not open.
   */
  public native int readStatus();

  /**
   * Close port
   * Does nothing, if port is not open.
   */
  public native void close();
}
