// ClipPlayer.java

import javax.sound.sampled.*;
import javax.swing.*;
import java.io.*;

public class ClipPlayer
{
  protected Clip clip = null;
  protected AudioFormat audioFormat = null;
  protected AudioInputStream audioInputStream = null;

  public ClipPlayer(AudioFormat audioFormat)
  {
    this.audioFormat = audioFormat;
  }

  public ClipPlayer(File audioFile)
  {
    try
    {
      audioInputStream =
        AudioSystem.getAudioInputStream(audioFile);
      audioFormat = audioInputStream.getFormat();
    }
    catch (Exception ex)
    {
      System.out.println(ex);
      System.exit(1);
    }
  }

  public void start(boolean doFinish)
          throws LineUnavailableException
  {
    if (audioInputStream == null)
      return;
    DataLine.Info dataLineInfo = new DataLine.Info(Clip.class,
                                   audioFormat);

    if (clip == null) // First invocation
    {
      clip = (Clip)AudioSystem.getLine(dataLineInfo);
      try
      {
        clip.open(audioInputStream);
      }
      catch (IOException ex)
      {
        System.out.println(ex);
        System.exit(1);
      }
    }
    else // replay same clip, put it at start
      clip.setFramePosition(0);
    clip.start();
    if (doFinish)
    {
      waitToFinish();
      clip.close();
    }
  }

  public boolean isPlaying()
  {
    if (clip == null)
      return false;
    return clip.isActive();
  }

  private void waitToFinish()
  {
    if (clip == null)
      return;
    while (clip.isActive())
    {
      try
      {
        Thread.currentThread().sleep(10);
      }
      catch (InterruptedException ex) {}
    }
  }

  public static void main(String[] args)
  {
    File file = null;
    JFileChooser chooser = new JFileChooser();
    int rc = chooser.showOpenDialog(null);
    if (rc == JFileChooser.APPROVE_OPTION)
      file = chooser.getSelectedFile();
    else
      System.exit(0);
    ClipPlayer player = new ClipPlayer(file);
    try
    {
      player.start(true);
    }
    catch (LineUnavailableException ex)
    {
      System.out.println("Soundkarte nicht verfgbar");
      System.exit(1);
    }
    System.out.println("All done");
    System.exit(0);
  }
}

