// FunctionPlayer.java

import javax.sound.sampled.*;
import java.io.*;

public class FunctionPlayer extends ClipPlayer
{
  private Waveform[] wf = {new Waveform.F0(),
                           new Waveform.F1(),
                           new Waveform.F2(),
                           new Waveform.F3(),
                           new Waveform.F4()};

  public FunctionPlayer(AudioFormat audioFormat)
  {
    super(audioFormat);
  }

  public void initData(int waveformIndex, double duration,
                       double frequency, double amplitude)
  {
    if (clip != null)
    {
      clip.close();
      clip = null;
    }
    ByteArrayOutputStream data = new ByteArrayOutputStream();
    float sampleRate = audioFormat.getSampleRate();
    int nbFrames = (int)(duration * sampleRate);
    amplitude = 32000 * amplitude;
    double t = 0;
    double dt = 1.0 / sampleRate;
    int nbZero = 2 * (int)(sampleRate / 2);
    int soundData;

    for (int i = 0; i < nbFrames; i++)
    {
      soundData = (int)(amplitude *
                        wf[waveformIndex].f(t, frequency));
      t += dt;
      byte lowLeft = (byte)(soundData & 0xFF);
      byte highLeft = (byte)((soundData >> 8) & 0xFF);
      data.write(lowLeft);
      data.write(highLeft);
    }

    InputStream is =
      new ByteArrayInputStream(data.toByteArray());
    audioInputStream =
      new AudioInputStream(is, audioFormat,
                           data.size() /
                           audioFormat.getFrameSize());
  }
}
