// Confetti.java

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;

public class Confetti extends JApplet implements Runnable
{
  private int width, height, size;
  private int delay; // in msec
  private Color color;
  private boolean isStandalone = false;
  private ConfettiPanel panel;
  private volatile boolean active = false;

  public String getParameter(String key, String def)
  {
    return isStandalone
           ? System.getProperty(key, def)
           : (getParameter(key) != null
              ? getParameter(key) : def);
  }

  public void init()
  {
    // Get parameters
    width = Integer.parseInt(getParameter("width", "500"));
    height = Integer.parseInt(getParameter("height", "500"));
    size = Integer.parseInt(getParameter("size", "20"));
    delay = Integer.parseInt(getParameter("delay", "10"));
    color = new Color(
      Integer.parseInt(getParameter("color", "FF0000"), 16));
    panel = new ConfettiPanel(width, height, size, color);
    getContentPane().add(panel);
  }

  public void initStandalone()
  {
    init();
    JFrame f = new JFrame();
    f.setTitle("Konfetti");
    f.setContentPane(getContentPane());
    f.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    f.pack();
    f.setResizable(false);
    f.setVisible(true);
    start();
  }

  public void start()
  {
    new Thread(this).start();
  }

  public void stop()
  {
    active = false; // Causes thread to stop
  }

  public void run()
  {
    active = true;
    if (!isStandalone)
      showStatus(" ");
    while (active)
    {
      try
      {
        Thread.sleep(delay);
      }
      catch (InterruptedException ex) {}
      panel.draw();
    }
  }

  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        Confetti confetti = new Confetti();
        confetti.isStandalone = true;
        confetti.initStandalone();
      }
    });
  }
}
