// PopDialog.java

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;

public class PopDialog extends JFrame
        implements ActionListener
{
  private final int DLG_WIDTH = 360;

  private State s;
  private JPanel contentPane;

  // -------------------- Panes ---------------------------
  private final int NB_PANES = 4;
  private JPanel[] panes = new JPanel[NB_PANES];
  private String[] paneTitles = {" Anfangswerte ",
                                 " Population ",
                                 " Simulation ", ""};
  private int[] paneHeights = {80, 155, 70, 20};

  // -------------------- TextFields ----------------------
  private final int NB_TEXTFIELDS = 2;
  private JTextField[] textFields =
    new JTextField[NB_TEXTFIELDS];

  // -------------------- Labels --------------------------
  private JLabel[] labels = new JLabel[NB_TEXTFIELDS];
  private String[] labelTexts = {"Anzahl Kranke (1..40): ",
                                 "Generationszeit (1..100): "};

  // -------------------- TextArea(s) ---------------------
  private JTextArea textArea = new JTextArea();
  private JScrollPane scrollPane =
    new JScrollPane(
      textArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

  // -------------------- Buttons -------------------------
  private final static int NB_BUTTONS = 4;
  private JButton[] buttons = new JButton[NB_BUTTONS];
  private String[] buttonTexts = {"Go", "Halt", "Reset",
                                  "Exit"};

  // -------------------- Status --------------------------
  private JLabel status = new JLabel();

  // -------------------- Menu ----------------------------
  private final static int NB_MENUS = 2;
  JMenu menus[] = new JMenu[NB_MENUS];
  private char[] menuMnemonics = {'D', 'S'};
  private String[] menuTexts = {"Datei", "Simulation"};

  private final static int NB_MENUITEMS = 4;
  private JMenuItem[] menuItems = new JMenuItem[NB_MENUITEMS];
  private String[] menuItemTexts = {"Exit", "Go", "Halt",
                                    "Reset"};
  private char[] menuItemAccelerators = {'X', 'G', 'H', 'R'};

  // -------------------- Constructor ---------------------
  public PopDialog(State s)
  {
    super("Ausbreitung einer Krankheit");
    this.s = s;
    setDefaultCloseOperation(
      WindowConstants.DO_NOTHING_ON_CLOSE);
    setJMenuBar(setMenu());
    initDialog();
    pack();
    setResizable(false);
    setLocation(200, 200);
    setVisible(true);
  }

  // -------------------- public Methods ------------------
  public void showText(String text)
  {
    textArea.append(text);
    scrollToEnd();
  }

  public void clear()
  {
    textArea.setText("");
  }

  public void showStatus(String text)
  {
    status.setText(" " + text);
  }

  public int getInputValue(int index, int min, int max)
  {
    IntegerValidator iv = new IntegerValidator(min, max);
    String value = textFields[index].getText().trim();
    if (iv.isValid(value))
    {
      return Integer.parseInt(value);
    }
    return 0;
  }

  public void setTextField(int index, String text)
  {
    textFields[index].setText(text);
  }

  public void enableInput(boolean b)
  {
    textFields[0].setEnabled(b);
    textFields[1].setEnabled(b);
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    for (int i = 0; i < NB_BUTTONS; i++)
    {
      if (source == buttons[i])
      {
        s.next(i);
        return;
      }
    }

    if (source == menuItems[0])
    {
      s.next(3);
      return;
    }

    for (int i = 1; i < NB_MENUITEMS; i++)
    {
      if (source == menuItems[i])
      {
        s.next(i - 1);
        return;
      }
    }
  }

  public void processWindowEvent(WindowEvent evt)
  {
    super.processWindowEvent(evt);
    if (evt.getID() == WindowEvent.WINDOW_CLOSING)
    {
      s.next(3);
    }
  }

  // -------------------- private Methods -----------------
  private void initDialog()
  {
    contentPane = (JPanel)getContentPane();
    contentPane.setLayout(new BoxLayout(contentPane,
                                        BoxLayout.Y_AXIS));
    GridBagLayout gridBag = new GridBagLayout();

    // ----------------- Panes ----------------------------
    for (int i = 0; i < NB_PANES; i++)
    {
      panes[i] = new JPanel();
      contentPane.add(panes[i]);
      panes[i].setPreferredSize(
        new Dimension(DLG_WIDTH, paneHeights[i]));
      if (i != NB_PANES - 1)
      {
        panes[i].setBorder(
          BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder(paneTitles[i]),
            BorderFactory.createEmptyBorder(5, 5, 5, 5)));
      }
    }

    panes[0].setLayout(gridBag);
    panes[0].setBackground(Color.white);

    panes[1].setLayout(new BorderLayout());
    panes[1].setBackground(Color.white);
    panes[1].add(scrollPane);

    panes[2].setLayout(new BoxLayout(panes[2],
                                     BoxLayout.X_AXIS));

    panes[3].setLayout(new BorderLayout());
    panes[3].setBorder(new BevelBorder(BevelBorder.LOWERED));
    panes[3].add(status);

    // ----------------- TextFields -----------------------
    for (int i = 0; i < NB_TEXTFIELDS; i++)
    {
      textFields[i] = new JTextField(10);
      labels[i] = new JLabel(labelTexts[i]);
    }
    addLabelTextRows(labels, textFields, gridBag, panes[0]);

    // ----------------- TextArea ------------------------
    textArea.setFont(new Font("Courier", Font.PLAIN, 14));

    // ----------------- Buttons --------------------------
    for (int i = 0; i < NB_BUTTONS; i++)
    {
      buttons[i] = new JButton(buttonTexts[i]);
      panes[2].add(buttons[i]);
      buttons[i].addActionListener(this);
    }

    /*
        // ----------------- AWTEventListener -----------------
        getToolkit().addAWTEventListener(
            new AWTEventListener()
        {
          public void eventDispatched(AWTEvent evt)
          {
            System.out.println(evt + "\n");
          }
        }

        ,
            AWTEvent.FOCUS_EVENT_MASK
    //      | AWTEvent.KEY_EVENT_MASK
    //      | AWTEvent.MOUSE_EVENT_MASK
            );
    */
  }

  private JMenuBar setMenu()
  {
    for (int i = 0; i < NB_MENUS; i++)
    {
      menus[i] = new JMenu(menuTexts[i]);
      menus[i].setMnemonic(menuMnemonics[i]);
    }

    menuItems[0] = new JMenuItem(menuItemTexts[0]);
    menuItems[0].setAccelerator(
      KeyStroke.getKeyStroke(
        menuItemAccelerators[0], InputEvent.ALT_MASK));

    menus[0].add(menuItems[0]);
    menuItems[0].addActionListener(this);


    for (int k = 1; k < NB_MENUITEMS; k++)
    {
      menuItems[k] = new JMenuItem(menuItemTexts[k]);
      menuItems[k].setAccelerator(
        KeyStroke.getKeyStroke(
          menuItemAccelerators[k], InputEvent.ALT_MASK));
      menus[1].add(menuItems[k]);
      menuItems[k].addActionListener(this);
    }

    JMenuBar menuBar = new JMenuBar();
    menuBar.add(menus[0]);
    menuBar.add(menus[1]);
    return menuBar;
  }

  private void scrollToEnd()
  {
    try
    {
      textArea.setCaretPosition(
        textArea.getLineEndOffset(
          textArea.getLineCount() - 1));
    }
    catch (BadLocationException ex)
    {
      System.out.println(ex);
    }
  }

  // Method from TextSamplerDemo.java
  // Sun, "The JFC Swing Tutorial"
  private void addLabelTextRows(JLabel[] labels,
                                JTextField[] textFields,
                                GridBagLayout gridBag,
                                Container container)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.EAST;
    int numLabels = labels.length;

    for (int i = 0; i < numLabels; i++)
    {
      c.gridwidth = GridBagConstraints.RELATIVE; //next-to-last
      c.fill = GridBagConstraints.NONE; //reset to default
      c.weightx = 0.0; //reset to default
      gridBag.setConstraints(labels[i], c);
      container.add(labels[i]);

      c.gridwidth = GridBagConstraints.REMAINDER; //end row
      c.fill = GridBagConstraints.HORIZONTAL;
      c.weightx = 1.0;
      gridBag.setConstraints(textFields[i], c);
      container.add(textFields[i]);
    }
  }
}
