// Star.java

import java.awt.geom.*;
import java.awt.*;
import ch.aplu.util.*;

public class Star
{
  private GPane panel;
  private int size;
  private Color color;
  private GeneralPath tri1, tri2; // 2 Dreiecke bei (0,0)

  // ------------- Konstructor ----------------------
  public Star(GPane panel, int radius, Color color)
  {
    panel.window(0, 500, 0, 500);
    this.panel = panel;
    this.color = color;
    setSize(radius);
  }

  // ---------- getter/setter Methoden ---------------
  public int getSize()
  {
    return size;
  }

  public void setSize(int size)
  {
    this.size = size;

    // Erstes Dreieck, size ist Umkreisradius, Mitte bei (0,0)
    tri1 = new GeneralPath();
    tri1.moveTo(size, 0);
    tri1.lineTo((int)(-0.5 * size), (int)(0.866 * size));
    tri1.lineTo((int)(-0.5 * size), (int)(-0.866 * size));
    tri1.closePath();

    // Zweites Dreieck, wie erstes aber um 180 Grad rotiert
    tri2 = (GeneralPath)tri1.clone();
    AffineTransform t = new AffineTransform();
    t.rotate(Math.PI);
    tri2.transform(t);
  }

  public Color getColor()
  {
    return color;
  }

  public void setColor(Color color)
  {
    this.color = color;
  }

  // ---------- public Methoden ---------------------
  public void turn(double angle)
  {
    AffineTransform t = new AffineTransform();
    t.rotate(angle);
    tri1.transform(t);
    tri2.transform(t);
  }

  public void showAt(int mx, int my)
  {
    panel.color(color);
    AffineTransform t = new AffineTransform();
    t.translate(mx, my);

    // Clonen, damit keine Seiteneffekte auftreten
    GeneralPath gp1 = (GeneralPath)tri1.clone();
    GeneralPath gp2 = (GeneralPath)tri2.clone();
    gp1.transform(t);
    gp2.transform(t);
    panel.fillGeneralPath(gp1);
    panel.fillGeneralPath(gp2);
  }

  // ---------- main() fr Tests --------------------
  public static void main(String[] args)
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        GPane p = new GPane(new Size(500, 500));
        Star s1 = new Star(p, 200, Color.red);
        s1.showAt(250, 250);
        Star s2 = new Star(p, 150, Color.green);
        s2.turn(Math.PI / 6);
        s2.showAt(250, 250);
      }
    });
  }
}
