// FibServer.java
// Create stub using rmic before execution

import java.util.*;
import java.rmi.server.*;
import java.rmi.registry.*;

public class FibServer implements FibIf
{
  private final static int port = 1099; // Default RMI port

  public FibServer()
  {
    Registry reg = null;
    try
    {
      // Check if registry is alive
      reg = LocateRegistry.getRegistry();
      reg.list();
    }
    catch (Exception ex)
    {
      reg = null;
    }
    try
    {
      if (reg == null) // instead of using "rmiregistry"
        reg = LocateRegistry.createRegistry(port);
      FibIf fib =
        (FibIf)UnicastRemoteObject.exportObject(this);
      reg.rebind("Fibonacci", fib);
      System.out.print("FibServer is running on port " +
                       port + " for 100 s... ");
      Thread.currentThread().sleep(100000);
      System.out.println(" and stopping now.");
      reg.unbind("Fibonacci");
      System.exit(0);
    }
    catch (Exception ex)
    {
      System.out.println("FibServer exception: " + ex);
      ex.printStackTrace();
      System.exit(1);
    }

  }

  public long getFib(int n)
  {
    long a = 1;
    long b = 1;

    if (n == 1)
      return a;
    if (n == 2)
      return b;
    for (int i = 0; i < n - 2; i++)
    {
      long temp = b;
      b = a + b;
      a = temp;
    }
    return b;
  }

  public Date getServerDate()
  {
    return new Date();
  }

  public static void main(String args[])
  {
    new FibServer();
  }
}
